@@ -34,31 +34,38 @@
 #include <sqlite3.h>
 #include "pkgin.h"
 
+/*
+ * Table name lookup, as a convenience for tables that have identical LOCAL_
+ * and REMOTE_ entities.
+ */
 static const struct Summary {
 	const int	type;
-	const char	*tbl_name;
-	const char	*deps;
+	const char	*pkg;
 	const char	*conflicts;
-	const char	*requires;
+	const char	*depends;
 	const char	*provides;
+	const char	*requires;
+	const char	*supersedes;
 	const char	*end;
 } sumsw[] = {
 	[LOCAL_SUMMARY] = {
 		LOCAL_SUMMARY,
 		"LOCAL_PKG",
-		"LOCAL_DEPS",
 		"LOCAL_CONFLICTS",
-		"LOCAL_REQUIRES",
+		"LOCAL_DEPENDS",
 		"LOCAL_PROVIDES",
+		"LOCAL_REQUIRES",
+		"LOCAL_SUPERSEDES",	/* Unused */
 		NULL
 	},
 	[REMOTE_SUMMARY] = {
 		REMOTE_SUMMARY,
 		"REMOTE_PKG",
-		"REMOTE_DEPS",
 		"REMOTE_CONFLICTS",
-		"REMOTE_REQUIRES",
+		"REMOTE_DEPENDS",
 		"REMOTE_PROVIDES",
+		"REMOTE_REQUIRES",
+		"REMOTE_SUPERSEDES",
 		NULL
 	},
 };
@@ -213,7 +220,7 @@ prepare_insert(int pkgid, struct Summary sum)
 	char		tmpbuf[1024];
 
 	sqlite3_snprintf(sizeof(querybuf), querybuf, "INSERT INTO %w (PKG_ID",
-	    sum.tbl_name);
+	    sum.pkg);
 
 	/* insert fields */
 	SLIST_FOREACH(pi, &inserthead, next) {
@@ -270,7 +277,7 @@ parse_entry(struct Summary sum, int pkgid, char *line)
 {
 	static uint8_t	check_machine_arch = 1;
 	int		i;
-	char		*val, *v, *pkg, buf[BUFSIZ];
+	char		*val, *v, buf[BUFSIZ];
 
 	if ((val = strchr(line, '=')) == NULL)
 		errx(EXIT_FAILURE, "Invalid pkg_info entry: %s", line);
@@ -292,35 +299,41 @@ parse_entry(struct Summary sum, int pkgid, char *line)
 		return;
 	}
 
-	/* CONFLICTS */
 	if (strncmp(line, "CONFLICTS=", 10) == 0) {
-		pkgindb_dovaquery(INSERT_SINGLE_VALUE, sum.conflicts,
-		    sum.conflicts, pkgid, val);
+		v = pkgname_from_pattern(val);
+		pkgindb_dovaquery(INSERT_CONFLICTS, sum.conflicts, pkgid, val,
+		    v);
+		free(v);
 		return;
 	}
 
-	/* DEPENDS */
 	if (strncmp(line, "DEPENDS=", 8) == 0) {
-		if ((pkg = get_pkgname_from_depend(val)) != NULL) {
-			pkgindb_dovaquery(INSERT_DEPENDS_VALUES, sum.deps,
-			    sum.deps, sum.deps, pkgid, pkg, val);
-			XFREE(pkg);
-		} else
-			printf(MSG_COULD_NOT_GET_PKGNAME, val);
+		v = pkgname_from_pattern(val);
+		pkgindb_dovaquery(INSERT_DEPENDS, sum.depends, pkgid, val, v);
+		free(v);
 		return;
 	}
 
-	/* REQUIRES */
-	if (strncmp(line, "REQUIRES=", 9) == 0) {
-		pkgindb_dovaquery(INSERT_SINGLE_VALUE, sum.requires,
-		    sum.requires, pkgid, val);
-		return;
-	}
-
-	/* PROVIDES */
 	if (strncmp(line, "PROVIDES=", 9) == 0) {
-		pkgindb_dovaquery(INSERT_SINGLE_VALUE, sum.provides,
-		    sum.provides, pkgid, val);
+		pkgindb_dovaquery(INSERT_PROVIDES, sum.provides, pkgid, val);
+		return;
+	}
+
+	if (strncmp(line, "REQUIRES=", 9) == 0) {
+		pkgindb_dovaquery(INSERT_REQUIRES, sum.requires, pkgid, val);
+		return;
+	}
+
+	if (strncmp(line, "SUPERSEDES=", 11) == 0) {
+		/*
+		 * Only remote SUPERSEDES are supported.
+		 */
+		if (sum.type == REMOTE_SUMMARY) {
+			v = pkgname_from_pattern(val);
+			pkgindb_dovaquery(INSERT_SUPERSEDES, sum.supersedes,
+			    pkgid, val, v);
+			free(v);
+		}
 		return;
 	}
 
@@ -384,7 +397,7 @@ insert_local_summary(FILE *fp)
 
 	/* record columns names to cols */
 	sqlite3_snprintf(BUFSIZ, buf, "PRAGMA table_info(%w);",
-	    sumsw[LOCAL_SUMMARY].tbl_name);
+	    sumsw[LOCAL_SUMMARY].pkg);
 	pkgindb_doquery(buf, colnames, NULL);
 
 	SLIST_INIT(&inserthead);
@@ -434,7 +447,7 @@ insert_remote_summary(struct archive *a, char *cur_repo)
 
 	/* record columns names to cols */
 	sqlite3_snprintf(buflen, buf, "PRAGMA table_info(%w);",
-	    sumsw[REMOTE_SUMMARY].tbl_name);
+	    sumsw[REMOTE_SUMMARY].pkg);
 	pkgindb_doquery(buf, colnames, NULL);
 
 	SLIST_INIT(&inserthead);
@@ -528,127 +541,203 @@ insert_remote_summary(struct archive *a, char *cur_repo)
 static void
 delete_remote_tbl(struct Summary sum, char *repo)
 {
-	const char	**arr;
+	const char **table;
 
 	/*
-	 * delete repository related tables
-	 * loop through sumsw structure to record table name
-	 * and call associated SQL query
+	 * Use results from REMOTE_PKG to delete entries from REMOTE_CONFLICTS
+	 * etc first, then finally remove from REMOTE_PKG.
 	 */
-	/* (REMOTE[LOCAL)_PKG is first -> skip */
-	for (arr = &(sum.tbl_name) + 1; *arr != NULL; ++arr) {
-		pkgindb_dovaquery(DELETE_REMOTE, *arr, *arr, *arr, *arr, *arr,
-		    repo, *arr);
+	for (table = &(sum.pkg) + 1; *table != NULL; ++table) {
+		pkgindb_dovaquery(DELETE_REMOTE, *table, repo);
 	}
-
 	pkgindb_dovaquery(DELETE_REMOTE_PKG_REPO, repo);
 }
 
+/*
+ * The following section is taken verbatim from pkg_install files/admin/main.c
+ * to correctly handle REQUIRED_BY entries.
+ */
+struct reqd_by_entry {
+	char *pkgname;
+	SLIST_ENTRY(reqd_by_entry) entries;
+};
+SLIST_HEAD(reqd_by_entry_head, reqd_by_entry);
+
+struct pkg_reqd_by {
+	char *pkgname;
+	struct reqd_by_entry_head required_by[PKG_HASH_SIZE];
+	SLIST_ENTRY(pkg_reqd_by) entries;
+};
+SLIST_HEAD(pkg_reqd_by_head, pkg_reqd_by);
+
 static void
-handle_manually_installed(Pkglist *pkglist, char **pkgkeep)
+add_required_by(const char *pattern, const char *pkgname, struct pkg_reqd_by_head *hash)
 {
-	/* update local db with manually installed packages */
-	if (pkgkeep == NULL && !is_automatic_installed(pkglist->full)) {
-		pkgindb_dovaquery(KEEP_PKG, pkglist->name);
+	struct pkg_reqd_by_head *phead;
+	struct pkg_reqd_by *pkg;
+	struct reqd_by_entry_head *ehead;
+	struct reqd_by_entry *entry;
+	char *best_installed;
+	int i;
+
+	best_installed = find_matching_installed_pkg(pattern, 1, 0);
+	if (best_installed == NULL) {
+		warnx("Dependency %s of %s unresolved", pattern, pkgname);
 		return;
 	}
-	mark_as_automatic_installed(pkglist->full, 1);
+	phead = &hash[PKG_HASH_ENTRY(best_installed)];
+	SLIST_FOREACH(pkg, phead, entries) {
+		if (strcmp(pkg->pkgname, best_installed) == 0) {
+			ehead = &pkg->required_by[PKG_HASH_ENTRY(pkgname)];
+			SLIST_FOREACH(entry, ehead, entries) {
+				if (strcmp(entry->pkgname, pkgname) == 0)
+					break;
+			}
+			if (entry == NULL) {
+				entry = xmalloc(sizeof(*entry));
+				entry->pkgname = xstrdup(pkgname);
+				SLIST_INSERT_HEAD(ehead, entry, entries);
+			}
+			break;
+		}
+	}
+	if (pkg == NULL) {
+		pkg = xmalloc(sizeof(*pkg));
+		pkg->pkgname = xstrdup(best_installed);
+		for (i = 0; i < PKG_HASH_SIZE; i++)
+			SLIST_INIT(&pkg->required_by[i]);
+		ehead = &pkg->required_by[PKG_HASH_ENTRY(pkgname)];
+		entry = xmalloc(sizeof(*entry));
+		entry->pkgname = xstrdup(pkgname);
+		SLIST_INSERT_HEAD(ehead, entry, entries);
+		SLIST_INSERT_HEAD(phead, pkg, entries);
+	}
+	free(best_installed);
+}
+
+static int
+add_depends_of(const char *pkgname, void *cookie)
+{
+	FILE *fp;
+	struct pkg_reqd_by_head *h = cookie;
+	plist_t *p;
+	package_t plist;
+	char *path;
+
+	path = pkgdb_pkg_file(pkgname, CONTENTS_FNAME);
+	if ((fp = fopen(path, "r")) == NULL)
+		errx(EXIT_FAILURE, "Cannot read %s of package %s",
+		    CONTENTS_FNAME, pkgname);
+	free(path);
+	read_plist(&plist, fp);
+	fclose(fp);
+
+	for (p = plist.head; p; p = p->next) {
+		if (p->type == PLIST_PKGDEP)
+			add_required_by(p->name, pkgname, h);
+	}
+
+	free_plist(&plist);
+
+	return 0;
 }
 
 static void
-update_localdb(char **pkgkeep)
+insert_local_required_by(void)
 {
-	FILE		*pinfo;
-	struct stat	st;
-	Plistnumbered	*keeplisthead, *nokeeplisthead;
-	Pkglist		*pkglist;
+	struct pkg_reqd_by_head pkgs[PKG_HASH_SIZE];
+	struct pkg_reqd_by *p;
+	struct reqd_by_entry *e;
+	int i, j;
+
+	for (i = 0; i < PKG_HASH_SIZE; i++)
+		SLIST_INIT(&pkgs[i]);
+
+	if (iterate_pkg_db(add_depends_of, &pkgs) == -1)
+		errx(EXIT_FAILURE, "cannot iterate pkgdb");
+
+	for (i = 0; i < PKG_HASH_SIZE; i++) {
+		SLIST_FOREACH(p, &pkgs[i], entries) {
+			for (j = 0; j < PKG_HASH_SIZE; j++) {
+				SLIST_FOREACH(e, &p->required_by[j], entries) {
+					pkgindb_dovaquery(INSERT_REQUIRED_BY,
+					    p->pkgname, e->pkgname);
+					free(e->pkgname);
+				}
+				while (!SLIST_EMPTY(&p->required_by[j])) {
+					e = SLIST_FIRST(&p->required_by[j]);
+					SLIST_REMOVE_HEAD(&p->required_by[j], entries);
+					free(e);
+				}
+			}
+			free(p->pkgname);
+		}
+		while (!SLIST_EMPTY(&pkgs[i])) {
+			p = SLIST_FIRST(&pkgs[i]);
+			SLIST_REMOVE_HEAD(&pkgs[i], entries);
+			free(p);
+		}
+	}
+}
+
+static void
+update_localdb(int verbose)
+{
+	FILE *pinfo;
+	struct stat st;
+	Pkglist *lpkg;
+	int l;
 
 	/*
-	 * Start a write transaction, excluding other writers until
-	 * committed.  This serves two purposes:
-	 *
-	 * - If interrupted before we've recreated the local summary
-	 *   and reinitialized the PKG_KEEP state, this ensures we
-	 *   don't lose anything.
-	 *
-	 * - If a concurrent pkgin process simultaneously tries to do
-	 *   update_localdb, it can't invalidate the keep list we read
-	 *   with KEEP_LOCAL_PKGS.
-	 *
-	 * Note that this does not nest inside transactions or
-	 * savepoints.
+	 * Start a write transaction, excluding other writers until committed.
 	 */
 	if (pkgindb_doquery("BEGIN IMMEDIATE;", NULL, NULL))
 		errx(EXIT_FAILURE, "failed to begin immediate transaction");
 
-	/* has the pkgdb (pkgsrc) changed ? if not, continue */
+	/*
+	 * Only replace the database if forced or if the pkgdb changed.
+	 */
 	if (!pkg_db_mtime(&st) && !force_fetch)
 		goto out;
 
-	/* record the keep list */
-	keeplisthead = rec_pkglist(KEEP_LOCAL_PKGS);
-
-	/* delete local pkg table (faster than updating) */
+	/*
+	 * Delete and recreate the LOCAL_PKG table, as it's simpler and faster
+	 * than updating.
+	 */
 	pkgindb_doquery(DELETE_LOCAL, NULL, NULL);
 
-	printf(MSG_READING_LOCAL_SUMMARY);
-	/* generate summary locally */
+	if (verbose)
+		printf(MSG_READING_LOCAL_SUMMARY);
+
 	if ((pinfo = popen(PKG_INSTALL_DIR "/pkg_info -Xa", "r")) == NULL)
 		errx(EXIT_FAILURE, "Couldn't run pkg_info");
 
-	printf(MSG_PROCESSING_LOCAL_SUMMARY);
+	if (verbose)
+		printf(MSG_PROCESSING_LOCAL_SUMMARY);
 
-	/* insert the summary to the database */
 	insert_local_summary(pinfo);
+	insert_local_required_by();
 	pkg_db_update_mtime(&st);
 	pclose(pinfo);
 
-	/* re-read local packages list as it may have changed */
-	free_global_pkglists();
-	init_global_pkglists();
+	/*
+	 * Reread the local package list.  This updates l_plisthead.
+	 */
+	free_local_pkglist();
+	init_local_pkglist();
 
-	/* restore keep-list */
-	if (keeplisthead != NULL) {
-		SLIST_FOREACH(pkglist, keeplisthead->P_Plisthead, next) {
-			pkgindb_dovaquery(KEEP_PKG, pkglist->name);
-		}
-		free_pkglist(&keeplisthead->P_Plisthead);
-		free(keeplisthead);
-
-		/*
-		 * packages are installed "manually" by pkgin_install()
-		 * they are recorded as "non-automatic" in pkgdb, we
-		 * need to mark unkeeps as "automatic".
-		 */
-		if ((nokeeplisthead = rec_pkglist(NOKEEP_LOCAL_PKGS)) != NULL) {
-			SLIST_FOREACH(	pkglist,
-					nokeeplisthead->P_Plisthead,
-					next) {
-				handle_manually_installed(pkglist, pkgkeep);
-			}
-
-			free_pkglist(&nokeeplisthead->P_Plisthead);
-			free(nokeeplisthead);
-		}
-	} else { /* empty keep list */
-		/*
-		 * no packages are marked as keep in pkgin's db
-		 * probably a fresh install or a rebuild
-		 * restore keep flags with pkgdb informations
-		 */
-		SLIST_FOREACH(pkglist, &l_plisthead, next) {
-			if (!is_automatic_installed(pkglist->full)) {
-				pkgindb_dovaquery(KEEP_PKG, pkglist->name);
-			}
+	/*
+	 * Insert PKG_KEEP database entries based on pkgdb data.
+	 */
+	for (l = 0; l < LOCAL_PKG_HASH_SIZE; l++) {
+	SLIST_FOREACH(lpkg, &l_plisthead[l], next) {
+		if (!is_automatic_installed(lpkg->full)) {
+			pkg_keep(KEEP, lpkg->full);
 		}
 	}
-
-	/* insert new keep list if there's any */
-	if (pkgkeep != NULL)
-		/* installation: mark the packages as "keep" */
-		pkg_keep(KEEP, pkgkeep);
-
-out:	if (pkgindb_doquery("COMMIT;", NULL, NULL))
+	}
+out:
+	if (pkgindb_doquery("COMMIT;", NULL, NULL))
 		errx(EXIT_FAILURE, "failed to commit transaction");
 }
 
@@ -727,13 +816,13 @@ update_remotedb(int verbose)
 }
 
 int
-update_db(int which, char **pkgkeep, int verbose)
+update_db(int which, int verbose)
 {
 	if (!have_privs(PRIVS_PKGINDB))
 		return EXIT_FAILURE;
 
 	/* always check for LOCAL_SUMMARY updates */
-	update_localdb(pkgkeep);
+	update_localdb(verbose);
 
 	if (which == REMOTE_SUMMARY)
 		update_remotedb(verbose);
